#import "MSCustomRouteViewController.h"

@interface MSCustomRouteViewController () <MRMapViewDelegate>
@property (nonatomic, strong) MRMapView *mapView;
@property (nonatomic, strong) MRDirections *directions;
@property (nonatomic, strong) MRPathOverlay *routeOverlay;
@end

@implementation MSCustomRouteViewController

- (id)initWithNibName:(NSString *)name bundle:(NSBundle *)bundle {
    if ((self = [super initWithNibName:name bundle:bundle])) {
        self.edgesForExtendedLayout = UIRectEdgeNone;
        
        UIBarButtonItem *button = [[UIBarButtonItem alloc] initWithTitle:@"Directions" style:UIBarButtonItemStylePlain target:self action:@selector(directionsAction)];
        self.navigationItem.rightBarButtonItems = @[button];
    }
    return self;
}

#pragma mark - UIViewController

- (void)loadView {
    self.mapView = [[MRMapView alloc] init];
    self.mapView.delegate = self;
    self.mapView.showsUserLocation = YES;
    self.mapView.showsRoutePath = NO;
    self.mapView.mapKey = [MREditorKey keyForMap:[MSExampleHost mapID] app:[MSExampleHost appID]];
    self.view = self.mapView;
}

#pragma mark - MSExampleViewController

+ (NSString *)exampleTitle {
    return @"Custom Route Line";
}

+ (NSString *)exampleInfo {
    return @"Example of how to customize the route path.\n\n"
    @"• `Directions` button to start and stop directions from the selected placemark.\n"
    @"• The default route path is now a red line of width 2.\n";
}

#pragma mark - MRMapViewDelegate

- (MRPathRenderer *)mapView:(MRMapView *)mapView rendererForOverlay:(MRPathOverlay *)overlay {
    MRPathRenderer *renderer = [[MRPathRenderer alloc] initWithOverlay:overlay];
    renderer.strokeColor = [UIColor redColor];
    renderer.lineCap = kCGLineCapRound;
    renderer.lineJoin = kCGLineJoinRound;
    renderer.lineWidth = 2;
    return renderer;
}

- (void)mapViewDidFinishLoadingMap:(MRMapView *)mapView {
    [self reloadRouteOverlay];
}

- (void)mapPickerDidPickMap:(nonnull MRMap *)map {
    self.mapView.mapKey = map.key;
}

- (nonnull UIViewController *)viewControllerToAddSheetToForMapView:(MRMapView *)mapView {
    return self;
}

#pragma mark - Action

- (void)directionsAction {
    if (!self.mapView.route && self.mapView.selectedAnnotation) {
        MRDirectionsRequest *request = [[MRDirectionsRequest alloc] init];
        request.app = self.mapView.mapKey.parent;
        request.destination = [MRDirectionsDestination destinationWithPlacemarkKey:((MRPlacemark *)self.mapView.selectedAnnotation).key];
        request.source = [MRDirectionsSource sourceWithCurrentLocation];
        
        __weak typeof(self) weakSelf = self;
        self.directions = [[MRDirections alloc] initWithRequest:request presentingViewController:self];
        [self.directions calculateDirectionsWithCompletionHandler:^(MRDirectionsResponse *response, NSError *error) {
            [weakSelf directionsResponseDidLoad:response error:error];
        }];
    } else {
        [self.directions cancel];
        self.directions = nil;
        [self.mapView setRoute:nil animated:YES];
        [self reloadRouteOverlay];
    }
}

#pragma mark - Internal

- (void)directionsResponseDidLoad:(MRDirectionsResponse *)response error:(NSError *)error {
    self.directions = nil;
    if (!error && response.routes.count > 0) {
        MRRoute *route = response.routes.firstObject;
        [self.mapView deselectAnnotationAnimated:NO];
        [self.mapView setRoute:route animated:YES];
        [self reloadRouteOverlay];
    } else {
        UIAlertController *alertController = [UIAlertController alertControllerWithTitle:@"Could not load directions" message:error.localizedDescription preferredStyle:UIAlertControllerStyleAlert];
        UIAlertAction *action = [UIAlertAction actionWithTitle:@"Ok" style:UIAlertActionStyleDefault handler:nil];
        [alertController addAction:action];
        [self presentViewController:alertController animated:YES completion:nil];
    }
}

- (void)reloadRouteOverlay {
    [self.mapView removeOverlay:self.routeOverlay];
    self.routeOverlay = nil;
    if (self.mapView.route) {
        UIBezierPath *path = [UIBezierPath bezierPath];
        for (MRRouteStep *i in self.mapView.route.steps) {
            if ([i.mapKey isEqualToKey:self.mapView.mapKey]) {
                [path appendPath:i.path];
            }
        }
        self.routeOverlay = [[MRPathOverlay alloc] initWithPath:path.CGPath];
        [self.mapView addOverlay:self.routeOverlay];
    }
}

@end
