#import "MSFriendManager.h"
#import "MSExampleViewController.h"

@implementation NSPointerArray (Helpers)

- (BOOL)containsPointer:(void *)value {
    for (NSInteger i = 0; i < self.count; i++) {
        if ([self pointerAtIndex:i] == value) {
            return YES;
        }
    }
    return NO;
}

- (void)removePointerIdenticalTo:(void *)value {
    for (NSInteger i = self.count-1; i >= 0; i--) {
        if ([self pointerAtIndex:i] == value) {
            [self removePointerAtIndex:i];
        }
    }
}

@end

@interface MSFriendManager ()
@property (nonatomic, strong) NSPointerArray *observers;
@property (nonatomic, strong) NSString *identifier;
@property (nonatomic, strong) NSMutableArray *pendingInvites;
@end

@implementation MSFriendManager

+ (instancetype)manager1 {
    static MSFriendManager *sSharedManager = nil;
    static dispatch_once_t sOnce = 0;
    dispatch_once(&sOnce, ^{
        sSharedManager = [[self alloc] initWithIdentifier:@"manager1"];
    });
    return sSharedManager;
}

+ (instancetype)manager2 {
    static MSFriendManager *sSharedManager = nil;
    static dispatch_once_t sOnce = 0;
    dispatch_once(&sOnce, ^{
        sSharedManager = [[self alloc] initWithIdentifier:@"manager2"];
    });
    return sSharedManager;
}

+ (instancetype)manager3 {
    static MSFriendManager *sSharedManager = nil;
    static dispatch_once_t sOnce = 0;
    dispatch_once(&sOnce, ^{
        sSharedManager = [[self alloc] initWithIdentifier:@"manager3"];
    });
    return sSharedManager;
}

+ (void)setActiveManager:(MSFriendManager *)manager {
    [[MSFriendManager manager1].session stopPostingLocationUpdates];
    [[MSFriendManager manager1].session stopListeningForUpdates];
    [[MSFriendManager manager2].session stopPostingLocationUpdates];
    [[MSFriendManager manager2].session stopListeningForUpdates];
    [[MSFriendManager manager3].session stopPostingLocationUpdates];
    [[MSFriendManager manager3].session stopListeningForUpdates];
    [manager.session startPostingLocationUpdates];
    [manager.session startListeningForUpdates];
}

- (id)initWithIdentifier:(NSString *)identifier {
    if ((self = [super init])) {
        self.observers = [NSPointerArray weakObjectsPointerArray];
        self.identifier = identifier;
        self.pendingInvites = [NSMutableArray array];
        
        if (self.friendKey) {
            __weak typeof(self) weakSelf = self;
            [MRSharingSession loginWithKey:self.friendKey password:self.password appKey:[MREditorKey keyWithIdentifier:[MSExampleHost appID]] completion:^(MRSharingSession *session, NSError *error){
                weakSelf.session = session;
            }];
        }
    }
    return self;
}

#pragma mark - Public

- (void)addObserver:(id<MSFriendManagerObserver>)observer {
    if (![self.observers containsPointer:(__bridge void *)observer]) {
        self.observers = [self.observers copy];
        [self.observers addPointer:(__bridge void *)observer];
    }
}

- (void)removeObserver:(id<MSFriendManagerObserver>)observer {
    self.observers = [self.observers copy];
    [self.observers removePointerIdenticalTo:(__bridge void *)observer];
}

- (void)newAccountWithPassword:(NSString *)password name:(NSString *)name {
    __weak typeof(self) weakSelf = self;
    [MRSharingSession createAccountWithPassword:password name:name appKey:[MREditorKey keyWithIdentifier:[MSExampleHost appID]] completion:^(MRSharingSession *session, NSError *error){
        __strong typeof(self) strongSelf = weakSelf;
        if (strongSelf) {
            if (error) {
                [strongSelf didFail:error];
            } else {
                strongSelf.friendKey = session.key;
                strongSelf.password = password;
                strongSelf.session = session;
                [strongSelf reloadData];
            }
        }
    }];
}

- (void)deleteAccount {
    __weak typeof(self) weakSelf = self;
    [self.session deleteAccountWithCompletion:^(NSError *error){
        __strong typeof(self) strongSelf = weakSelf;
        if (strongSelf && !error) {
            strongSelf.friendKey = nil;
            strongSelf.password = nil;
            strongSelf.session = nil;
            strongSelf.profile = nil;
            strongSelf.friends = nil;
            strongSelf.invites = nil;
            [strongSelf reloadData];
        }
    }];
}

- (void)acceptInviteURL:(NSURL *)url {
    [self.pendingInvites addObject:url];
    [self reloadData];
}

- (void)reloadData {
    if (self.session) {
        __weak typeof(self) weakSelf = self;
        [MRFriend getAllFriendsWithSession:self.session completion:^(NSArray *friends, NSError *error){
            __strong typeof(self) strongSelf = weakSelf;
            if (strongSelf && !error) {
                strongSelf.friends = friends;
                [strongSelf didUpdate];
            }
        }];
        [MRInvite getAllInvitesWithSession:self.session completion:^(NSArray *invites, NSError *error){
            __strong typeof(self) strongSelf = weakSelf;
            if (strongSelf && !error) {
                strongSelf.invites = invites;
                [strongSelf didUpdate];
            }
        }];
        [MRFriend getFriendWithKey:self.session.key session:self.session completion:^(MRFriend *profile, NSError *error){
            __strong typeof(self) strongSelf = weakSelf;
            if (strongSelf && !error) {
                strongSelf.profile = profile;
                [strongSelf didUpdate];
            }
        }];
        for (NSURL *i in self.pendingInvites) {
            [MRInvite acceptInviteWithURL:i session:self.session completion:^(MRFriend *friend, NSError *error){
                __strong typeof(self) strongSelf = weakSelf;
                if (strongSelf && !error) {
                    [strongSelf reloadData];
                }
            }];
        }
    }

    self.pendingInvites = nil;
    [self didUpdate];
}

#pragma mark - Internal

- (void)setSession:(MRSharingSession *)value {
    if (value != _session) {
        _session = value;
        [self reloadData];
    }
}

- (void)didUpdate {
    for (id<MSFriendManagerObserver> i in self.observers) {
        if ([i respondsToSelector:@selector(friendManagerDidUpdate:)]) {
            [i friendManagerDidUpdate:self];
        }
    }
}

- (void)didFail:(NSError *)error {
    for (id<MSFriendManagerObserver> i in self.observers) {
        if ([i respondsToSelector:@selector(friendManagerDidFailWithError:)]) {
            [i friendManagerDidFailWithError:error];
        }
    }
}

- (NSString *)password {
    return [[NSUserDefaults standardUserDefaults] stringForKey:[NSString stringWithFormat:@"%@ password", self.identifier]];
}

- (void)setPassword:(NSString *)value {
    [[NSUserDefaults standardUserDefaults] setObject:value forKey:[NSString stringWithFormat:@"%@ password", self.identifier]];
}

- (MREditorKey *)friendKey {
    NSString *string = [[NSUserDefaults standardUserDefaults] stringForKey:[NSString stringWithFormat:@"%@ key", self.identifier]];
    return string ? [MREditorKey keyWithIdentifier:string] : nil;
}

- (void)setFriendKey:(MREditorKey *)value {
    [[NSUserDefaults standardUserDefaults] setObject:value.identifier forKey:[NSString stringWithFormat:@"%@ key", self.identifier]];
}

@end

@implementation UIViewController (MSFriendManager)

- (void)handleFriendError:(NSError *)error {
    UIAlertController *alert = [UIAlertController alertControllerWithTitle:@"Error" message:error.localizedDescription preferredStyle:UIAlertControllerStyleAlert];
    [alert addAction:[UIAlertAction actionWithTitle:@"OK" style:UIAlertActionStyleDefault handler:nil]];
    [self presentViewController:alert animated:YES completion:nil];
}

@end
